<?php

class GridGalleryPro_Galleries_Model_Galleries extends GridGallery_Galleries_Model_Galleries
{
	/**
	 * Extends default gallery select
	 *
	 * @param object $gallery The result of the SELECT
	 * @return mixed
	 */
	protected function extend($gallery, $attributes = array())
	{
       	if (!is_object($gallery)) {
            if ($this->debugEnabled) {
                wp_die(sprintf('The $gallery variable must be type of object, %s given', gettype($gallery)));
            }

            return $gallery;
        }

		$settingsModel = new GridGallery_Galleries_Model_Settings($this->debugEnabled);
		$settings = $settingsModel->getByGalleryId($gallery->id);
		if(is_object($settings)) {
			$gallery->settings_id = $settings->id;
			$gallery->settings = $settings->data;
		}

		if(isset($attributes) && isset($attributes['membershipModel'])) {
			$memberShipModel = $attributes['membershipModel'];
			foreach($attributes['image-list'] as $oneImage) {
				$gallery->photos[] = $memberShipModel->getGalleryAttachmenEmuledImage($oneImage);
			}
		} else {

			$resourcesModel = new GridGalleryPro_Galleries_Model_Resources($this->debugEnabled);

			if(isset($settings->data['showMore']) && $settings->data['showMore']['enabled'] === 'true' && property_exists($this, 'frontend')) {
				$limit = $settings->data['showMore']['imagesAmount'];
				if($settings->data['categories']['enabled'] !== 'true') {
					// #821
//					$resourcesData = $resourcesModel->showMore($gallery->id, $limit);
					$resourcesData = $resourcesModel->showMore($gallery->id);
				} else {
					$resourcesData = $resourcesModel->getByGalleryId($gallery->id);
				}
			} else {
				$resourcesData = $resourcesModel->getByGalleryId($gallery->id);
			}

			if(!$resourcesData) {
				return $gallery;
			}

			$photosModel = new GridGalleryPro_Photos_Model_Photos($this->debugEnabled);
			$photosModel->setGallerySettings(is_object($settings) ? $settings : false);
			$gallery->photos = $photosModel->getPhotos($resourcesData);

			// Place here because we need categories order both in frontend and backend.
			$gallery = $this->setCategoriesOrder($gallery);

			//Get all tags from gallery
			$tagsModel = new GridGalleryPro_Galleries_Model_Tags($this->debugEnabled);
			$gallery->tags = $tagsModel->getAllTags($gallery->id);

			if(isset($gallery->settings['attributes']) && isset($gallery->settings['attributes']['order']) && sizeof($gallery->settings['attributes']['order']) > 0) {
				$attributesModel = new GridGalleryPro_Galleries_Model_Attributes($this->debugEnabled);
				$gallery->attributes = $attributesModel->getAllAttributesValues($gallery->photos);
			}
		}

		return $gallery;
	}

	/**
	 * Set order of the categories
	 * @param object $gallery 
	 * @return object $gallery
	 */
	public function setCategoriesOrder($gallery)
	{
		$settingsModel = new GridGallery_Galleries_Model_Settings($this->debugEnabled); 

		// If not set any categories
		if (!isset($gallery->settings['categories']['order'])) {
		   $gallery->settings['categories']['order'] = array('__all__');
		}

		$settings = $gallery->settings;

		$categories = array('__all__');

		if (property_exists($gallery, 'photos')) {
			foreach ($gallery->photos as $photo) {
				if (property_exists($photo, 'tags') && is_array($photo->tags) && count($photo->tags) > 0) {
					foreach ($photo->tags as $tag) {
						$categories[] = $tag;
					}
				}
			}
		}

		// Add categories from posts or pages if enable.
		if (isset($settings['posts']) && $settings['posts']['enable']) {

			foreach ($settingsModel->getPostsToRender($gallery->id) as $post) {
				foreach($post['categories'] as $category) {
					$categories[] = $category['name'];
				}
			}

			foreach ($settingsModel->getPagesToRender($gallery->id) as $page) {
				foreach($page['categories'] as $category) {
					$categories[] = $category['name'];
				}
			}
		}

		// All stored categories in db for this gallery
		$categories = array_unique($categories);

		// Merge saved categories with new categories in db if new exist
		$merged = array_merge(
			$settings['categories']['order'], 
			array_diff(
				$categories,
				$settings['categories']['order']
			)
		);

		// Left only those categories that exist in db and filter empty
		$gallery->settings['categories']['order'] = array_filter(array_intersect(
			$merged,
			$categories
		));

		return $gallery;
	}

	public function cloneGallery($galleryId, $cloneType, $parameters) {
		$language = $parameters['environment']->getLang();
		$tagsModel = $parameters['tags'];
		$hasError = false;
		$message = '';
		$res1 = parent::cloneGallery($galleryId, $cloneType, $parameters);
		if(isset($res1['message']) && isset($res1['isError']) && isset($res1['newGalleryId'])) {
			$message = $res1['message'];
			$hasError = $res1['isError'];
			$newGalleryId = $res1['newGalleryId'];

			if($cloneType != 2) {
				// all tags attached to image
			}
		} else {
			$message = $language->translate('Pro plugin params Error!');
			$hasError = true;
		}

		return array(
			'message' => $message,
			'isError' => $hasError,
			'newGalleryId' => $newGalleryId,
		);
	}
}
